from salmonella.widgets import SalmonellaIdWidget, SalmonellaMultiIdWidget


class SalmonellaMixin(object):
    salmonella_fields = ()

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if db_field.name in self.salmonella_fields:
            kwargs['widget'] = SalmonellaIdWidget(db_field.rel, self.admin_site)
            return db_field.formfield(**kwargs)
        return super(SalmonellaMixin, self).formfield_for_foreignkey(
            db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        if db_field.name in self.salmonella_fields:
            kwargs['widget'] = SalmonellaMultiIdWidget(db_field.rel, self.admin_site)
            kwargs['help_text'] = ''
            return db_field.formfield(**kwargs)
        return super(SalmonellaMixin, self).formfield_for_manytomany(db_field,
                                                                     request,
                                                                     **kwargs)



from django.core.urlresolvers import reverse
from django.test.testcases import TestCase
from django.contrib.auth.models import User

from salmonella.tests.testapp.models import (CharPrimaryKeyModel,
                                             DirectPrimaryKeyModel,
                                             SalmonellaTest)

class SalmonellaTestCase(TestCase):
    """
    Test the basic integrity of the app. We can't test the Javascript side
    and all those fancy popups, but we can load an admin view and check that
    Salmonella was successfully loaded and displays items properly.
    """
    def setUp(self):
        self.admin = User.objects.create_superuser('admin', '', 'admin')
        self.client.login(username='admin', password='admin')

    def tearDown(self):
        self.client.logout()

    def get_labelview_url(self, multi=False):
        name = multi and 'salmonella_multi_label' or 'salmonella_label'
        return reverse(name, kwargs={
            'app_name': 'testapp', 'model_name': 'salmonellatest'
        })

    def create_sample_data(self):
        """
        Create a bit of sample data we can use to assign.
        """
        self.u1 = User.objects.create_superuser('jon', 'jon@example.com', '')
        self.u2 = User.objects.create_superuser('jim', 'jim@example.com', '')
        self.c1 = CharPrimaryKeyModel.objects.create(chr='helloworld')
        self.n1 = DirectPrimaryKeyModel.objects.create(num=12345)

        self.obj = SalmonellaTest.objects.create(
            rawid_fk=self.u1,
            rawid_fk_limited=self.u1,
            rawid_fk_direct_pk=self.n1,
            salmonella_fk=self.u1,
            salmonella_fk_limited=self.u1,
            salmonella_fk_direct_pk=self.n1,
            salmonella_fk_char_pk=self.c1,
        )
        self.obj.rawid_many.add(self.u1, self.u2)
        self.obj.salmonella_many.add(self.u1, self.u2)

    def test_changelist_integrity(self):
        """
        The `SalmonellaFilter` is hooked up in the right filter bar of the
        testapp changelist view.
        """
        self.create_sample_data()
        list_url = reverse('admin:testapp_salmonellatest_changelist')
        response = self.client.get(list_url)
        self.assertEqual(response.status_code, 200)

    def test_change_integrity(self):
        """
        The `SalmonellaFilter` is hooked up in the right filter bar of the
        testapp changelist view.
        """
        self.create_sample_data()
        list_url = reverse('admin:testapp_salmonellatest_change', args=(self.obj.pk,))
        response = self.client.get(list_url)
        self.assertEqual(response.status_code, 200)

    def test_labelview_unauthed(self):
        """
        Label view required authentication and a staff account
        """
        self.client.logout()
        response = self.client.get(self.get_labelview_url(), follow=True)
        self.assertEqual(response.status_code, 404)

    def test_labelview(self):
        """
        Call the labelview directly (what usually an Ajax JS call would do)
        and check for proper response.
        """
        self.create_sample_data()
        response = self.client.get(self.get_labelview_url(), {'id': self.obj.pk},
                                   follow=True)
        self.assertEqual(response.status_code, 200)

    def test_multi_labelview(self):
        """
        Call the labelview directly (what usually an Ajax JS call would do)
        and check for proper response.
        """
        self.create_sample_data()
        response = self.client.get(self.get_labelview_url(multi=True),
                                   {'id': self.obj.pk}, follow=True)
        self.assertEqual(response.status_code, 200)

    def test_invalid_id(self):
        """
        Test invalid id.
        """
        self.create_sample_data()
        response = self.client.get(self.get_labelview_url(), {'id': 'wrong'},
                                   follow=True)
        self.assertEqual(response.status_code, 400)

    def test_no_id(self):
        """
        Test invalid app/model name.
        """
        self.create_sample_data()
        response = self.client.get(self.get_labelview_url(), follow=True)
        self.assertEqual(response.status_code, 400)

    def test_invalid_appname(self):
        """
        Test invalid app/model name.
        """
        self.create_sample_data()
        url = self.get_labelview_url().replace('testapp', 'foobar')
        response = self.client.get(url, {'id': self.obj.pk}, follow=True)
        self.assertEqual(response.status_code, 400)



# coding: utf-8

"""Salmonella filters."""

from django import forms
from django.contrib import admin
from salmonella.widgets import SalmonellaIdWidget


class SalmonellaFilterForm(forms.Form):

    """Form for Salmonella filter."""

    def __init__(self, rel, admin_site, field_name, **kwargs):
        """Construct field for given field rel."""
        super(SalmonellaFilterForm, self).__init__(**kwargs)

        self.fields['%s' % field_name] = forms.IntegerField(
            label='', widget=SalmonellaIdWidget(
                rel=rel, admin_site=admin_site), required=False)


class SalmonellaFilter(admin.filters.FieldListFilter):

    """Filter list queryset by primary key of related object."""

    template = 'salmonella/admin/filters/salmonella_filter.html'

    def __init__(self, field, request, params, model, model_admin, field_path):
        """Use GET param for lookup and form initialization."""
        self.lookup_kwarg = '%s' % field_path
        super(SalmonellaFilter, self).__init__(
            field, request, params, model, model_admin, field_path)
        self.form = self.get_form(request, field.rel, model_admin.admin_site)

    def choices(self, cl):
        """Filter choices are not available."""
        return []

    def expected_parameters(self):
        """Return GET params for this filter."""
        return [self.lookup_kwarg]

    def get_form(self, request, rel, admin_site):
        """Return filter form."""
        return SalmonellaFilterForm(admin_site=admin_site, rel=rel,
                                    field_name=self.field_path,
                                    data=self.used_parameters)

    def queryset(self, request, queryset):
        """Filter queryset using params from the form."""
        if self.form.is_valid():
            # get no null params
            filter_params = dict(filter(lambda x: bool(x[1]),
                                        self.form.cleaned_data.items()))
            return queryset.filter(**filter_params)
        else:
            return queryset



from django.db import models

class DirectPrimaryKeyModel(models.Model):
    num = models.IntegerField("Number", primary_key=True)

    def __str__(self):
        return str(self.num)

class CharPrimaryKeyModel(models.Model):
    chr = models.CharField(max_length=20, primary_key=True)

    def __str__(self):
        return self.chr

class SalmonellaTest(models.Model):
    rawid_fk = models.ForeignKey('auth.User',
        related_name='rawid_fk', blank=True, null=True)

    rawid_fk_limited = models.ForeignKey('auth.User',
        related_name='rawid_fk_limited',
        limit_choices_to={'is_staff': True},
        blank=True, null=True)

    rawid_many = models.ManyToManyField('auth.User',
        related_name='rawid_many', blank=True)

    rawid_fk_direct_pk = models.ForeignKey(DirectPrimaryKeyModel,
        related_name='rawid_fk_direct_pk', blank=True, null=True)

    salmonella_fk = models.ForeignKey('auth.User',
        related_name='salmonella_fk', blank=True, null=True)

    salmonella_fk_limited = models.ForeignKey('auth.User',
        related_name='salmonella_fk_limited',
        limit_choices_to={'is_staff': True},
        blank=True, null=True)

    salmonella_many = models.ManyToManyField('auth.User',
        related_name='salmonella_many', blank=True)

    salmonella_fk_direct_pk = models.ForeignKey(DirectPrimaryKeyModel,
        related_name='salmonella_fk_direct_pk', blank=True, null=True)

    salmonella_fk_char_pk = models.ForeignKey(CharPrimaryKeyModel,
            related_name='salmonella_fk_char_pk', blank=True, null=True)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-08 18:21
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CharPrimaryKeyModel',
            fields=[
                ('chr', models.CharField(max_length=20, primary_key=True, serialize=False)),
            ],
        ),
        migrations.CreateModel(
            name='DirectPrimaryKeyModel',
            fields=[
                ('num', models.IntegerField(primary_key=True, serialize=False, verbose_name=b'Number')),
            ],
        ),
        migrations.CreateModel(
            name='SalmonellaTest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rawid_fk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='rawid_fk', to=settings.AUTH_USER_MODEL)),
                ('rawid_fk_direct_pk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='rawid_fk_direct_pk', to='testapp.DirectPrimaryKeyModel')),
                ('rawid_fk_limited', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='rawid_fk_limited', to=settings.AUTH_USER_MODEL)),
                ('rawid_many', models.ManyToManyField(blank=True, null=True, related_name='rawid_many', to=settings.AUTH_USER_MODEL)),
                ('salmonella_fk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='salmonella_fk', to=settings.AUTH_USER_MODEL)),
                ('salmonella_fk_char_pk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='salmonella_fk_char_pk', to='testapp.CharPrimaryKeyModel')),
                ('salmonella_fk_direct_pk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='salmonella_fk_direct_pk', to='testapp.DirectPrimaryKeyModel')),
                ('salmonella_fk_limited', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='salmonella_fk_limited', to=settings.AUTH_USER_MODEL)),
                ('salmonella_many', models.ManyToManyField(blank=True, null=True, related_name='salmonella_many', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.contrib import admin

from salmonella.admin import SalmonellaMixin
from salmonella.filters import SalmonellaFilter

from .models import SalmonellaTest, DirectPrimaryKeyModel, CharPrimaryKeyModel


class SalmonellaTestAdmin(SalmonellaMixin, admin.ModelAdmin):
    raw_id_fields = (
        'rawid_fk',
        'rawid_fk_limited',
        'rawid_many'
    )
    salmonella_fields = (
        'salmonella_fk',
        'salmonella_fk_limited',
        'salmonella_many',
        'salmonella_fk_direct_pk',
        'salmonella_fk_char_pk'
    )
    list_filter = (
        ('salmonella_fk', SalmonellaFilter),
    )

admin.site.register(DirectPrimaryKeyModel)
admin.site.register(CharPrimaryKeyModel)
admin.site.register(SalmonellaTest, SalmonellaTestAdmin)



from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponseBadRequest, HttpResponseForbidden
from django.shortcuts import render_to_response
from django.apps import apps


@user_passes_test(lambda u: u.is_staff)
def label_view(request, app_name, model_name, template_name="", multi=False,
               template_object_name="object"):

    # The list of to obtained objects is in GET.id. No need to resume if we
    # didnt get it.
    if not request.GET.get('id'):
        msg = 'No list of objects given'
        return HttpResponseBadRequest(settings.DEBUG and msg or '')

    # Given objects are either an integer or a comma-separted list of
    # integers. Validate them and ignore invalid values. Also strip them
    # in case the user entered values by hand, such as '1, 2,3'.
    object_list = []
    for pk in request.GET['id'].split(","):
        object_list.append(pk.strip())

    # Check if at least one value survived this cleanup.
    if len(object_list) == 0:
        msg = 'No list or only invalid ids of objects given'
        return HttpResponseBadRequest(settings.DEBUG and msg or '')

    # Make sure this model exists and the user has 'change' permission for it.
    # If he doesnt have this permission, Django would not display the
    # change_list in the popup and the user were never able to select objects.
    try:
        model = apps.get_model(app_name, model_name)
    except LookupError:
        msg = 'Model %s.%s does not exist.' % (app_name, model_name)
        return HttpResponseBadRequest(settings.DEBUG and msg or '')

    if not request.user.has_perm('%s.change_%s' % (app_name, model_name)):
        return HttpResponseForbidden()

    try:
        if multi:
            model_template = "salmonella/%s/multi_%s.html" % (app_name, model_name)
            objs = model.objects.filter(pk__in=object_list)
            objects = []
            for obj in objs:
                change_url = reverse("admin:%s_%s_change" % (app_name, model_name),
                                     args=[obj.pk])
                obj = (obj, change_url)
                objects.append(obj)
            extra_context = {
                template_object_name: objects,
            }
        else:
            model_template = "salmonella/%s/%s.html" % (app_name, model_name)
            obj = model.objects.get(pk=object_list[0])
            change_url = reverse("admin:%s_%s_change" % (app_name, model_name),
                                 args=[obj.pk])
            extra_context = {
                template_object_name: (obj, change_url),
            }
    # most likely the pk wasn't convertable
    except ValueError:
        msg = 'ValueError during lookup'
        return HttpResponseBadRequest(settings.DEBUG and msg or '')
    except model.DoesNotExist:
        msg = 'Model instance does not exist'
        return HttpResponseBadRequest(settings.DEBUG and msg or '')

    return render_to_response((model_template, template_name), extra_context)



from django.conf.urls import *
from salmonella.views import label_view

urlpatterns = [
    url(r'^(?P<app_name>[\w-]+)/(?P<model_name>[\w-]+)/multiple/$',
        label_view,
        {
            'multi': True,
            'template_object_name': 'objects',
            'template_name': 'salmonella/multi_label.html'
        },
        name="salmonella_multi_label"),
    url(r'^(?P<app_name>[\w-]+)/(?P<model_name>[\w-]+)/$',
        label_view,
        {
            'template_name': 'salmonella/label.html'
        },
        name="salmonella_label"),
]



#!/usr/bin/env python
from setuptools import find_packages, setup

setup(
    name='django-salmonella',
    version='1.1',
    author='Lincoln Loop: Seth Buntin, Yann Malet',
    author_email='info@lincolnloop.com',
    description=("raw_id_fields widget replacement that handles display of an object's "
                 "string value on change and can be overridden via a template."),
    long_description=open('README.rst').read(),
    url='https://github.com/lincolnloop/django-salmonella/',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Framework :: Django',
    ],
    packages=find_packages(),
    package_data={
        'salmonella': ['templates/*.*', 'static/*.*'],
        'docs': ['*'],
    },
    include_package_data=True,
    install_requires=[
        'django>=1.8',
    ],
)



from django.conf import settings
from django.contrib.admin import widgets
from django.core.urlresolvers import reverse, NoReverseMatch
from django.core.exceptions import ImproperlyConfigured
from django.template.loader import render_to_string
from django import VERSION

try:
    from django.utils.encoding import force_text
except ImportError:
    from django.utils.encoding import force_unicode as force_text


class SalmonellaImproperlyConfigured(ImproperlyConfigured):
    pass


class SalmonellaIdWidget(widgets.ForeignKeyRawIdWidget):
    template_name = 'salmonella/admin/widgets/salmonella_field_dj111.html'

    def render(self, name, value, attrs=None, multi=False):
        """
        Django <= 1.10 variant.
        """
        if VERSION[0] >= 1 and VERSION[1] >= 11:
            return super(SalmonellaIdWidget, self).render(name, value, attrs, renderer=None)

        if attrs is None:
            attrs = {}

        try:
            related_url = reverse('admin:%s_%s_changelist' % (
                self.rel.to._meta.app_label,
                self.rel.to._meta.object_name.lower()),
                current_app=self.admin_site.name)
        except NoReverseMatch:
            raise SalmonellaImproperlyConfigured('The model %s.%s is not '
                'registered in the admin.' % (self.rel.to._meta.app_label,
                                              self.rel.to._meta.object_name))

        params = self.url_parameters()
        if params:
            url = u'?' + u'&'.join([u'%s=%s' % (k, v) for k, v in params.items()])
        else:
            url = u''
        if "class" not in attrs:
            attrs['class'] = 'vForeignKeyRawIdAdminField'  # The JavaScript looks for this hook.
        app_name = self.rel.to._meta.app_label.strip()
        model_name = self.rel.to._meta.object_name.lower().strip()
        hidden_input = super(widgets.ForeignKeyRawIdWidget, self).render(name, value, attrs)

        extra_context = {
            'hidden_input': hidden_input,
            'name': name,
            'app_name': app_name,
            'model_name': model_name,
            'related_url': related_url,
            'url': url,
            'SALMONELLA_STATIC': settings.STATIC_URL + 'salmonella/'
        }
        return render_to_string('salmonella/admin/widgets/salmonella_field.html',
                                extra_context)

    def get_context(self, name, value, attrs):
        """
        Django >= 1.11 variant.
        """
        context = super(SalmonellaIdWidget, self).get_context(name, value, attrs)

        try:
            related_url = reverse('admin:%s_%s_changelist' % (
                self.rel.to._meta.app_label,
                self.rel.to._meta.object_name.lower()),
                current_app=self.admin_site.name)
        except NoReverseMatch:
            raise SalmonellaImproperlyConfigured('The model %s.%s is not '
                'registered in the admin.' % (self.rel.to._meta.app_label,
                                              self.rel.to._meta.object_name))

        params = self.url_parameters()
        if params:
            url = u'?' + u'&'.join([u'%s=%s' % (k, v) for k, v in params.items()])
        else:
            url = u''
        if "class" not in attrs:
            attrs['class'] = 'vForeignKeyRawIdAdminField'  # The JavaScript looks for this hook.
        app_name = self.rel.to._meta.app_label.strip()
        model_name = self.rel.to._meta.object_name.lower().strip()

        context.update({
            'name': name,
            'app_name': app_name,
            'model_name': model_name,
            'related_url': related_url,
            'url': url,
            'SALMONELLA_STATIC': settings.STATIC_URL + 'salmonella/'
        })
        return context

    class Media:
        js = (settings.STATIC_URL + "salmonella/js/salmonella.js",)


class SalmonellaMultiIdWidget(SalmonellaIdWidget):
    def value_from_datadict(self, data, files, name):
        value = data.get(name)
        if value:
            return value.split(',')

    def render(self, name, value, attrs):
        attrs['class'] = 'vManyToManyRawIdAdminField'
        value = ','.join([force_text(v) for v in value]) if value else ''
        return super(SalmonellaMultiIdWidget, self).render(name, value, attrs)
